import "core-js/modules/es6.promise";
import "core-js/modules/es7.symbol.async-iterator";
import "core-js/modules/es6.symbol";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.regexp.search";
import "core-js/modules/es6.string.starts-with";
import "core-js/modules/es6.regexp.match";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
import getAPIVersion from './getAPIVersion';
/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */

/**
 * Cache content using the service worker.  If content is not supplied, the service worker will fetch
 * the content from the server
 * @param {String} path The URI path of the request
 * @param {String} cacheData The data to cache
 */

export function cache(_x, _x2) {
  return _cache.apply(this, arguments);
}
/**
 * Returns the cached response for the given URL if present.
 * @param {String} cacheName The name of the cache to pull from
 * @param {String} url A json fetch URL
 * @return {Promise<Response>}
 */

function _cache() {
  _cache = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(path, cacheData) {
    var _ref, apiVersion;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context.sent) {
              _context.next = 7;
              break;
            }

            _ref = window.moov || {}, apiVersion = _ref.apiVersion;

            if (!(window.moov.router && !window.moov.router.willCacheOnClient({
              path: path
            }))) {
              _context.next = 6;
              break;
            }

            return _context.abrupt("return");

          case 6:
            if (cacheData) {
              navigator.serviceWorker.controller.postMessage({
                action: 'cache-state',
                path: path,
                apiVersion: apiVersion,
                cacheData: cacheData
              });
            } else {
              navigator.serviceWorker.controller.postMessage({
                action: 'cache-path',
                path: path,
                apiVersion: apiVersion
              });
            }

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _cache.apply(this, arguments);
}

export function getCachedResponse(_x3, _x4) {
  return _getCachedResponse.apply(this, arguments);
}

function _getCachedResponse() {
  _getCachedResponse = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(cacheName, url) {
    var cachedResponse;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return caches.open("".concat(cacheName, "-").concat(getAPIVersion())).then(function (cache) {
              return cache.match(new Request(url));
            });

          case 2:
            cachedResponse = _context2.sent;
            console.log('cachedResponse', cacheName, url, cachedResponse);
            return _context2.abrupt("return", cachedResponse);

          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return _getCachedResponse.apply(this, arguments);
}

if (typeof window !== 'undefined') {
  window._moovFromCache = getCachedResponse;
}
/**
 * Prefetches and caches JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */


export function prefetchJsonFor(_x5, _x6) {
  return _prefetchJsonFor.apply(this, arguments);
}
/**
 * Prefetches and caches SSR and JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */

function _prefetchJsonFor() {
  _prefetchJsonFor = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee3(path, includeSSR) {
    var url, _url;

    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (path) {
              _context3.next = 2;
              break;
            }

            return _context3.abrupt("return");

          case 2:
            if (path.startsWith('http')) {
              url = new URL(path);
              cache("".concat(url.origin).concat(url.pathname, ".json").concat(url.search));
            } else {
              _url = new URL("http://z.z".concat(path));
              cache("".concat(_url.pathname, ".json").concat(_url.search));
            }

            if (includeSSR) {
              cache(path);
            }

          case 4:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return _prefetchJsonFor.apply(this, arguments);
}

export function prefetch(path) {
  cache(path);
  prefetchJsonFor(path);
}
/**
 * Aborts all in progress prefetches.  Call this function to prevent prefetching from blocking
 * more important requests, like page navigation.
 */

export function abortPrefetches() {
  return _abortPrefetches.apply(this, arguments);
}
/**
 * Resume queued prefetch requests which were cancelled to allow for more important requests
 */

function _abortPrefetches() {
  _abortPrefetches = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee4() {
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context4.sent) {
              _context4.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({
              action: 'abort-prefetches'
            });

          case 4:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));
  return _abortPrefetches.apply(this, arguments);
}

export function resumePrefetches() {
  return _resumePrefetches.apply(this, arguments);
}
/**
 * Configures runtime caching options
 * @private
 * @param {Object} options
 * @param {Object} options.cacheName The name of the runtime cache
 * @param {Object} options.maxEntries The max number of entries to store in the cache
 * @param {Object} options.maxAgeSeconds The TTL in seconds for entries
 */

function _resumePrefetches() {
  _resumePrefetches = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee5() {
    return _regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context5.sent) {
              _context5.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({
              action: 'resume-prefetches'
            });

          case 4:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));
  return _resumePrefetches.apply(this, arguments);
}

export function configureCache(_x7) {
  return _configureCache.apply(this, arguments);
}
/**
 * Clears all API and SSR responses from the client cache
 * @return {Promise} Resolved once all caches have b
 */

function _configureCache() {
  _configureCache = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee6(options) {
    return _regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context6.sent) {
              _context6.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({
              action: 'configure-runtime-caching',
              options: options
            });

          case 4:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));
  return _configureCache.apply(this, arguments);
}

export function clearCache() {
  return _clearCache.apply(this, arguments);
}
/**
 * Resolves when the service worker has been installed
 * @private
 */

function _clearCache() {
  _clearCache = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee7() {
    var keys, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, key;

    return _regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            if (!('caches' in window)) {
              _context7.next = 31;
              break;
            }

            _context7.next = 3;
            return caches.keys();

          case 3:
            keys = _context7.sent;
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context7.prev = 7;
            _iterator = keys[Symbol.iterator]();

          case 9:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context7.next = 17;
              break;
            }

            key = _step.value;

            if (key.startsWith('workbox-precache')) {
              _context7.next = 14;
              break;
            }

            _context7.next = 14;
            return caches.delete(key);

          case 14:
            _iteratorNormalCompletion = true;
            _context7.next = 9;
            break;

          case 17:
            _context7.next = 23;
            break;

          case 19:
            _context7.prev = 19;
            _context7.t0 = _context7["catch"](7);
            _didIteratorError = true;
            _iteratorError = _context7.t0;

          case 23:
            _context7.prev = 23;
            _context7.prev = 24;

            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }

          case 26:
            _context7.prev = 26;

            if (!_didIteratorError) {
              _context7.next = 29;
              break;
            }

            throw _iteratorError;

          case 29:
            return _context7.finish(26);

          case 30:
            return _context7.finish(23);

          case 31:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7, this, [[7, 19, 23, 31], [24,, 26, 30]]);
  }));
  return _clearCache.apply(this, arguments);
}

export function waitForServiceWorkerController() {
  return _waitForServiceWorkerController.apply(this, arguments);
}
/**
 * Removes runtime caches for old versions of the api.  This ensures that all responses
 * are appropriate for the current version of the UI.
 * @private
 */

function _waitForServiceWorkerController() {
  _waitForServiceWorkerController = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee8() {
    return _regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            if (!(!navigator.serviceWorker || !navigator.serviceWorker.ready)) {
              _context8.next = 2;
              break;
            }

            return _context8.abrupt("return", false);

          case 2:
            return _context8.abrupt("return", new Promise(function (resolve) {
              navigator.serviceWorker.ready.then(function () {
                if (navigator.serviceWorker.controller) {
                  return resolve(true);
                }

                navigator.serviceWorker.addEventListener('controllerchange', function () {
                  return resolve(true);
                });
              });
            }));

          case 3:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8, this);
  }));
  return _waitForServiceWorkerController.apply(this, arguments);
}

export function removeOldCaches() {
  return _removeOldCaches.apply(this, arguments);
}

function _removeOldCaches() {
  _removeOldCaches = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee9() {
    return _regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context9.sent) {
              _context9.next = 4;
              break;
            }

            if (window.moov && window.moov.apiVersion) {
              navigator.serviceWorker.controller.postMessage({
                action: 'remove-old-caches',
                apiVersion: window.moov.apiVersion
              });
            }

          case 4:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9, this);
  }));
  return _removeOldCaches.apply(this, arguments);
}

export default {
  removeOldCaches: removeOldCaches,
  waitForServiceWorkerController: waitForServiceWorkerController,
  configureCache: configureCache,
  resumePrefetches: resumePrefetches,
  abortPrefetches: abortPrefetches,
  prefetch: prefetch,
  prefetchJsonFor: prefetchJsonFor,
  cache: cache
};
//# sourceMappingURL=serviceWorker.js.map